"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RTCConnection = void 0;
const tgcalls_1 = require("./tgcalls");
const binding_1 = require("./binding");
class RTCConnection {
    constructor(chat_id, file_path, binding, bitrate, buffer_length, invite_hash) {
        this.chat_id = chat_id;
        this.file_path = file_path;
        this.binding = binding;
        this.bitrate = bitrate;
        this.buffer_length = buffer_length;
        this.invite_hash = invite_hash;
        this.tgcalls = new tgcalls_1.TGCalls({ chat_id });
        this.stream = new tgcalls_1.Stream(file_path, 16, bitrate, 1, buffer_length);
        this.tgcalls.joinVoiceCall = async (payload) => {
            payload = {
                chat_id: this.chat_id,
                ufrag: payload.ufrag,
                pwd: payload.pwd,
                hash: payload.hash,
                setup: payload.setup,
                fingerprint: payload.fingerprint,
                source: payload.source,
                invite_hash: this.invite_hash,
            };
            binding_1.Binding.log('callJoinPayload -> ' + JSON.stringify(payload), binding_1.Binding.INFO);
            const joinCallResult = await this.binding.sendUpdate({
                action: 'join_voice_call_request',
                payload: payload
            });
            binding_1.Binding.log('joinCallRequestResult -> ' + JSON.stringify(joinCallResult), binding_1.Binding.INFO);
            return joinCallResult;
        };
        this.stream.on('finish', async () => {
            await this.binding.sendUpdate({
                action: 'stream_ended',
                chat_id: chat_id,
            });
        });
        this.stream.on('stream_deleted', async () => {
            this.stream.stop();
            await this.binding.sendUpdate({
                action: 'update_request',
                result: 'STREAM_DELETED',
                chat_id: chat_id,
            });
        });
    }
    async joinCall() {
        try {
            let result = await this.tgcalls.start(this.stream.createTrack());
            this.stream.resume();
            return result;
        }
        catch (e) {
            this.stream.stop();
            binding_1.Binding.log('joinCallError -> ' + e.toString(), binding_1.Binding.INFO);
            return false;
        }
    }
    stop() {
        try {
            this.stream.stop();
            this.tgcalls.close();
        }
        catch (e) { }
    }
    async leave_call() {
        try {
            this.stop();
            return await this.binding.sendUpdate({
                action: 'leave_call_request',
                chat_id: this.chat_id,
            });
        }
        catch (e) {
            return {
                action: 'REQUEST_ERROR',
                message: e.toString(),
            };
        }
    }
    pause() {
        this.stream.pause();
    }
    async resume() {
        this.stream.resume();
    }
    changeStream(file_path) {
        this.stream.setReadable(file_path);
    }
}
exports.RTCConnection = RTCConnection;
